/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Intalio, Inc.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Intalio, Inc. Exolab is a registered
 *    trademark of Intalio, Inc.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY INTALIO, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * INTALIO, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Intalio, Inc. All Rights Reserved.
 *
 * $Id: Token.java,v 1.1.1.1 2003/03/03 07:08:25 kvisco Exp $
 */


package org.exolab.castor.jdo.oql;

/**
 * Represents tokens which are generated by the {@link Lexer} from the String 
 * OQL representation.  Tokens contain a token type and the string value.
 *
 * @author <a href="nissim@nksystems.com">Nissim Karpenstein</a>
 * @version $Revision: 1.1.1.1 $ $Date: 2003/03/03 07:08:25 $
 */
public class Token implements TokenTypes {

  private int _tokenType;
  private String _tokenValue;

  /**
   * Creates a new Token with the supplied type and value.
   *
   * @param tokenType The Token Type for this token
   * @param tokenValue The Token Value for this token
   */
  public Token(int tokenType, String tokenValue) {
    _tokenType = tokenType;
    _tokenValue = tokenValue;
  }

  public String toString() {
    return _tokenValue; 
  }

  /**
   * Token Type accessor method.
   *
   * @return The Token Type for this token
   */
  public int getTokenType() {
    return _tokenType;
  }

  /**
   * Token value accessor method.
   *
   * @return The Token Value for this token
   */
  public String getTokenValue() {
    return _tokenValue;
  }

  

}
